==> uswds_ckeditor_integration-2.0.3.tar.gz: uswds_ckeditor_integration/js/ckeditor5_plugins/uswds_grid/src/utils.js <==
import { isWidget } from "ckeditor5/src/widget";

/**
 * Checks if the provided model element is `uswdsGrid`.
 *
 * @param {module:engine/model/element~Element} modelElement
 *   The model element to be checked.
 * @return {boolean}
 *   A boolean indicating if the element is a uswdsGrid element.
 *
 * @private
 */
export function isUswdsGrid(modelElement) {
  return !!modelElement && modelElement.is("element", "uswdsGrid");
}

/**
 * Checks if the provided model element is `uswdsGridContainer`.
 *
 * @param {module:engine/model/element~Element} modelElement
 *   The model element to be checked.
 * @return {boolean}
 *   A boolean indicating if the element is a uswdsGridContainer element.
 *
 * @private
 */
export function isUswdsGridContainer(modelElement) {
  return !!modelElement && modelElement.is("element", "uswdsGridContainer");
}

/**
 * Checks if view element is <uswdsGrid> element.
 *
 * @param {module:engine/view/element~Element} viewElement
 *   The view element.
 * @return {boolean}
 *   A boolean indicating if the element is a <uswdsGrid> element.
 *
 * @private
 */
export function isUswdsGridWidget(viewElement) {
  return isWidget(viewElement) && !!viewElement.getCustomProperty("uswdsGrid");
}

/**
 * Checks if view element is <uswdsGridContainer> element.
 *
 * @param {module:engine/view/element~Element} viewElement
 *   The view element.
 * @return {boolean}
 *   A boolean indicating if the element is a <uswdsGridContainer> element.
 *
 * @private
 */
export function isUswdsGridContainerWidget(viewElement) {
  return (
    isWidget(viewElement) && !!viewElement.getCustomProperty("uswdsGridContainer")
  );
}

/**
 * Checks if view element is <uswdsGridRow> element.
 *
 * @param {module:engine/view/element~Element} viewElement
 *   The view element.
 * @return {boolean}
 *   A boolean indicating if the element is a <uswdsGridRow> element.
 *
 * @private
 */
export function isUswdsGridRowWidget(viewElement) {
  return isWidget(viewElement) && !!viewElement.getCustomProperty("uswdsGridRow");
}

/**
 * Checks if view element is <uswdsGridCol> element.
 *
 * @param {module:engine/view/element~Element} viewElement
 *   The view element.
 * @return {boolean}
 *   A boolean indicating if the element is a <uswdsGridCol> element.
 *
 * @private
 */
export function isUswdsGridColWidget(viewElement) {
  return !!viewElement.getCustomProperty("uswdsGridCol");
}

/**
 * Gets `uswdsGrid` element from selection.
 *
 * @param {module:engine/model/selection~Selection|module:engine/model/documentselection~DocumentSelection} selection
 *   The current selection.
 * @return {module:engine/model/element~Element|null}
 *   The `uswdsGrid` element which could be either the current selected an
 *   ancestor of the selection. Returns null if the selection has no Grid
 *   element.
 *
 * @private
 */
export function getClosestSelectedUswdsGridElement(selection) {
  const selectedElement = selection.getSelectedElement();

  return isUswdsGrid(selectedElement)
    ? selectedElement
    : selection.getFirstPosition().findAncestor("uswdsGrid");
}

/**
 * Gets selected BsGrid widget if only BsGrid is currently selected.
 *
 * @param {module:engine/model/selection~Selection} selection
 *   The current selection.
 * @return {module:engine/view/element~Element|null}
 *   The currently selected Grid widget or null.
 *
 * @private
 */
export function getClosestSelectedUswdsGridWidget(selection) {
  const viewElement = selection.getSelectedElement();
  if (viewElement && isUswdsGridWidget(viewElement)) {
    return viewElement;
  }
  let { parent } = selection.getFirstPosition();
  while (parent) {
    if (parent.is("element") && isUswdsGridWidget(parent)) {
      return parent;
    }
    parent = parent.parent;
  }
  return null;
}

/**
 * Extracts classes for settings.
 *
 * @param {*} element
 *   The element being passed.
 * @param {string} base
 *   The base class to exclude.
 * @param {boolean} reverse
 *   Whether to reverse the affect.
 * @return {string|string|string}
 *   The class list.
 */
export function extractGridClasses(element, base, reverse) {
  reverse = reverse || false;
  let classes = "";

  if (typeof element.getAttribute === "function") {
    classes = element.getAttribute("class");
  }
 else if (typeof element.className === "string") {
    classes = element.className;
  }

  // Failsafe.
  if (!classes) {
    return "";
  }

  const classlist = classes.split(" ").filter((c) => {
    if (
      c.lastIndexOf("ck-widget", 0) === 0 ||
      c.lastIndexOf("ck-edit", 0) === 0 ||
      c.lastIndexOf("uswdsg-", 0) === 0
    ) {
      return false;
    }
    return reverse
      ? c.lastIndexOf(base, 0) === 0
      : c.lastIndexOf(base, 0) !== 0;
  });

  return classlist.length ? classlist.join(" ").trim() : "";
}

/**
 * Converts a grid into a settings object.
 *
 * @param {module:engine/view/element~Element|null} grid
 *   The current grid.
 * @return {{}}
 *   The settings.
 */
export function convertGridToSettings(grid) {
  const settings = {};
  let row = false;

  settings.container_wrapper_class = extractGridClasses(grid, "uswds_grid");

  // First child might be container or row.
  const firstChild = grid.getChild(0);

  // Container.
  if (isUswdsGridContainerWidget(firstChild)) {
    settings.add_container = 1;
    settings.container_class = extractGridClasses(firstChild, "container");

    // Container can have no classes, so need direct compare.
    const containerType = extractGridClasses(firstChild, "container", true);
    if (containerType.length) {
      if (containerType.indexOf("container-fluid") !== -1) {
        settings.container_type = "fluid";
      }
      else {
        settings.container_type = "default";
      }
    }
    row = firstChild.getChild(0);
  }
  else {
    row = firstChild;
  }

  // Row options.
  const rowClasses = extractGridClasses(row, "row");
  settings.no_gutter = rowClasses.indexOf("no-gutters") !== -1 ? 1 : 0;
  settings.row_class = rowClasses.replace("no-gutters", "").replace("g-0", "");

  // Layouts.
  settings.breakpoints = {
    none: { layout: row.getAttribute("data-row-none") },
    sm: { layout: row.getAttribute("data-row-sm") },
    md: { layout: row.getAttribute("data-row-md") },
    lg: { layout: row.getAttribute("data-row-lg") },
    xl: { layout: row.getAttribute("data-row-xl") },
    xxl: { layout: row.getAttribute("data-row-xxl") },
  };

  // Col options.
  settings.num_columns = 0;
  Array.from(row.getChildren()).forEach((col, idx) => {
    if (isUswdsGridColWidget(col)) {
      settings.num_columns += 1;
      const colClass = extractGridClasses(col, "col");
      const key = `col_${idx + 1}_classes`;
      settings[key] = colClass;
    }
  });

  return settings;
}

==> uswds_ckeditor_integration-3.1.2.tar.gz: uswds_ckeditor_integration/js/ckeditor5_plugins/uswds_grid/src/utils.js <==
import { isWidget } from 'ckeditor5/src/widget';

/**
 * Checks if the provided model element is `uswdsGrid`.
 *
 * @param {module:engine/model/element~Element} modelElement
 *   The model element to be checked.
 * @return {boolean}
 *   A boolean indicating if the element is a uswdsGrid element.
 *
 * @private
 */
export function isUswdsGrid(modelElement) {
  return !!modelElement && modelElement.is('element', 'uswdsGrid');
}

/**
 * Checks if the provided model element is `uswdsGridContainer`.
 *
 * @param {module:engine/model/element~Element} modelElement
 *   The model element to be checked.
 * @return {boolean}
 *   A boolean indicating if the element is a uswdsGridContainer element.
 *
 * @private
 */
export function isUswdsGridContainer(modelElement) {
  return !!modelElement && modelElement.is('element', 'uswdsGridContainer');
}

/**
 * Checks if view element is <uswdsGrid> element.
 *
 * @param {module:engine/view/element~Element} viewElement
 *   The view element.
 * @return {boolean}
 *   A boolean indicating if the element is a <uswdsGrid> element.
 *
 * @private
 */
export function isUswdsGridWidget(viewElement) {
  return isWidget(viewElement) && !!viewElement.getCustomProperty('uswdsGrid');
}

/**
 * Checks if view element is <uswdsGridContainer> element.
 *
 * @param {module:engine/view/element~Element} viewElement
 *   The view element.
 * @return {boolean}
 *   A boolean indicating if the element is a <uswdsGridContainer> element.
 *
 * @private
 */
export function isUswdsGridContainerWidget(viewElement) {
  return (
    isWidget(viewElement) &&
    !!viewElement.getCustomProperty('uswdsGridContainer')
  );
}

/**
 * Checks if view element is <uswdsGridRow> element.
 *
 * @param {module:engine/view/element~Element} viewElement
 *   The view element.
 * @return {boolean}
 *   A boolean indicating if the element is a <uswdsGridRow> element.
 *
 * @private
 */
export function isUswdsGridRowWidget(viewElement) {
  return (
    isWidget(viewElement) && !!viewElement.getCustomProperty('uswdsGridRow')
  );
}

/**
 * Checks if view element is <uswdsGridCol> element.
 *
 * @param {module:engine/view/element~Element} viewElement
 *   The view element.
 * @return {boolean}
 *   A boolean indicating if the element is a <uswdsGridCol> element.
 *
 * @private
 */
export function isUswdsGridColWidget(viewElement) {
  return !!viewElement.getCustomProperty('uswdsGridCol');
}

/**
 * Gets `uswdsGrid` element from selection.
 *
 * @param {module:engine/model/selection~Selection|module:engine/model/documentselection~DocumentSelection} selection
 *   The current selection.
 * @return {module:engine/model/element~Element|null}
 *   The `uswdsGrid` element which could be either the current selected an
 *   ancestor of the selection. Returns null if the selection has no Grid
 *   element.
 *
 * @private
 */
export function getClosestSelectedUswdsGridElement(selection) {
  const selectedElement = selection.getSelectedElement();

  return isUswdsGrid(selectedElement)
    ? selectedElement
    : selection.getFirstPosition().findAncestor('uswdsGrid');
}

/**
 * Gets selected BsGrid widget if only BsGrid is currently selected.
 *
 * @param {module:engine/model/selection~Selection} selection
 *   The current selection.
 * @return {module:engine/view/element~Element|null}
 *   The currently selected Grid widget or null.
 *
 * @private
 */
export function getClosestSelectedUswdsGridWidget(selection) {
  const viewElement = selection.getSelectedElement();
  if (viewElement && isUswdsGridWidget(viewElement)) {
    return viewElement;
  }

  // Nothing Selected.
  if (selection.getFirstPosition() === null) {
    return null;
  }

  let { parent } = selection.getFirstPosition();
  while (parent) {
    if (parent.is('element') && isUswdsGridWidget(parent)) {
      return parent;
    }
    parent = parent.parent;
  }
  return null;
}

/**
 * Extracts classes for settings.
 *
 * @param {*} element
 *   The element being passed.
 * @param {string} base
 *   The base class to exclude.
 * @param {boolean} reverse
 *   Whether to reverse the affect.
 * @return {string|string|string}
 *   The class list.
 */
export function extractGridClasses(element, base, reverse) {
  reverse = reverse || false;
  let classes = '';

  if (typeof element.getAttribute === 'function') {
    classes = element.getAttribute('class');
  }
 else if (typeof element.className === 'string') {
    classes = element.className;
  }

  // Failsafe.
  if (!classes) {
    return '';
  }

  const classlist = classes.split(' ').filter((c) => {
    if (
      c.lastIndexOf('ck-widget', 0) === 0 ||
      c.lastIndexOf('ck-edit', 0) === 0 ||
      c.lastIndexOf('uswdsg-', 0) === 0
    ) {
      return false;
    }
    return reverse
      ? c.lastIndexOf(base, 0) === 0
      : c.lastIndexOf(base, 0) !== 0;
  });

  return classlist.length ? classlist.join(' ').trim() : '';
}

/**
 * Converts a grid into a settings object.
 *
 * @param {module:engine/view/element~Element|null} grid
 *   The current grid.
 * @return {{}}
 *   The settings.
 */
export function convertGridToSettings(grid) {
  const settings = {};
  let row = false;

  settings.container_wrapper_class = extractGridClasses(grid, 'uswds_grid');

  // First child might be container or row.
  const firstChild = grid.getChild(0);

  // Container.
  if (isUswdsGridContainerWidget(firstChild)) {
    settings.add_container = 1;
    settings.container_class = extractGridClasses(firstChild, 'container');

    // Container can have no classes, so need direct compare.
    const containerType = extractGridClasses(firstChild, 'container', true);
    if (containerType.length) {
      if (containerType.indexOf('container-fluid') !== -1) {
        settings.container_type = 'fluid';
      }
      else {
        settings.container_type = 'default';
      }
    }
    row = firstChild.getChild(0);
  }
  else {
    row = firstChild;
  }

  // Row options.
  const rowClasses = extractGridClasses(row, 'row');
  settings.no_gutter = rowClasses.indexOf('no-gutters') !== -1 ? 1 : 0;
  settings.row_class = rowClasses.replace('no-gutters', '').replace('g-0', '');

  // Layouts.
  settings.breakpoints = {
    none: { layout: row.getAttribute('data-row-none') },
    sm: { layout: row.getAttribute('data-row-sm') },
    md: { layout: row.getAttribute('data-row-md') },
    lg: { layout: row.getAttribute('data-row-lg') },
    xl: { layout: row.getAttribute('data-row-xl') },
    xxl: { layout: row.getAttribute('data-row-xxl') },
  };

  // Col options.
  settings.num_columns = 0;
  Array.from(row.getChildren()).forEach((col, idx) => {
    if (isUswdsGridColWidget(col)) {
      settings.num_columns += 1;
      const colClass = extractGridClasses(col, 'col');
      const key = `col_${idx + 1}_classes`;
      settings[key] = colClass;
    }
  });

  return settings;
}
